<cfcomponent extends="shape" output="false">

	<cfset BOX_WIDTH = 120 />
	<cfset TOP_MARGIN = 10 />
	<cfset LEFT_MARGIN = 10 />
	<cfset CENTER_SPACE = 20 />
	<cfset RIGHT_MARGIN = 10 />
	<cfset BOTTOM_MARGIN = 15 />
	
	<cffunction name="init" access="public" output="false" returntype="shape">
		<cfargument name="verb" type="struct" required="false" />
		<cfset super.init(verb) />
		<cfset variables.my.trueList = createObject("component", "shapelist").init(verb.trueVerbs) />
		<cfset variables.my.falseList = createObject("component", "shapelist").init(verb.falseVerbs) />
		<cfreturn this />
	</cffunction>
	


	<cffunction name="draw" access="public" output="false" returntype="void">
		<cfargument name="img" type="any" required="true" />
		<cfset var m = measure() />
		<cfset var t = my.trueList.measure() />
		<cfset var f = my.falseList.measure() />
		<cfset var shape = "" />
		<cfset var txt = my.verb.type />
		<cfset var tOffset = -1 * (CENTER_SPACE / 2 + max(BOX_WIDTH, t.width) / 2) />
		<cfset var fOffset = CENTER_SPACE / 2 + max(BOX_WIDTH, f.width) / 2 />
		<cfset var wasTrueTerminated = my.trueList.isFlowTerminated() />
		<cfset var wasFalseTerminated = my.falseList.isFlowTerminated() />
		<!--- this balances uneven sides of the condition --->
		<cfset var shift = max(BOX_WIDTH, t.width) - m.width / 2 + LEFT_MARGIN + CENTER_SPACE / 2 />
<!---
<cfset msg("#txt# (#m.width# x #m.height#)") />
<cfset imageSetDrawingColor(img, "9999ff") />
<cfset imageDrawRect(img, m.width / -2, 0, m.width, m.height) />
--->
		<cfset resetColors(img) />
		
		<cfif isFlowTerminated()>
			<cfset my.trueList.setFlowTerminated(true) />
			<cfset my.falseList.setFlowTerminated(true) />
		</cfif>
		
		<!--- box --->
		<cfset imageDrawLine(img, 0, 0, BOX_WIDTH / 2, BOX_HEIGHT / 2) />
		<cfset imageDrawLine(img, BOX_WIDTH / 2, BOX_HEIGHT / 2, 0, BOX_HEIGHT) />
		<cfset imageDrawLine(img, 0, BOX_HEIGHT, BOX_WIDTH / -2, BOX_HEIGHT / 2) />
		<cfset imageDrawLine(img, BOX_WIDTH / -2, BOX_HEIGHT / 2, 0, 0) />
		<cfset imageDrawText(img, txt, getTextWidth(txt) / -2, (BOX_HEIGHT + getTextHeight()) / 2) />
		<cfset imageTranslateDrawingAxis(img, shift, 0) />
		<!--- top left (true) arrow --->
		<cfset imageDrawText(img, "true", tOffset - getTextWidth("true") - 10, (BOX_HEIGHT + getTextHeight()) / 2) />
		<cfset imageDrawLine(img, BOX_WIDTH / -2 - shift, BOX_HEIGHT / 2, tOffset, BOX_HEIGHT / 2) />
		<cfset imageDrawLine(img, tOffset, BOX_HEIGHT / 2, tOffset, BOX_HEIGHT + TOP_MARGIN) />
		<cfif NOT my.trueList.isEmpty()>
			<!--- arrowhead --->
			<cfset imageDrawLine(img, tOffset, BOX_HEIGHT + TOP_MARGIN, tOffset + 10, BOX_HEIGHT) />
			<cfset imageDrawLine(img, tOffset, BOX_HEIGHT + TOP_MARGIN, tOffset - 10, BOX_HEIGHT) />
		</cfif>
		<!--- top right (false) arrow --->
		<cfset imageDrawText(img, "false", fOffset + 10, (BOX_HEIGHT + getTextHeight()) / 2) />
		<cfset imageDrawLine(img, BOX_WIDTH / 2 - shift, BOX_HEIGHT / 2, fOffset, BOX_HEIGHT / 2) />
		<cfset imageDrawLine(img, fOffset, BOX_HEIGHT / 2, fOffset, BOX_HEIGHT + TOP_MARGIN) />
		<cfif NOT my.falseList.isEmpty()>
			<!--- arrowhead --->
			<cfset imageDrawLine(img, fOffset, BOX_HEIGHT + TOP_MARGIN, fOffset - 10, BOX_HEIGHT) />
			<cfset imageDrawLine(img, fOffset, BOX_HEIGHT + TOP_MARGIN, fOffset + 10, BOX_HEIGHT) />
		</cfif>
		
		<cfset imageTranslateDrawingAxis(img, tOffset, BOX_HEIGHT + TOP_MARGIN) />
		<cfset my.trueList.draw(img) />
		<cfset imageTranslateDrawingAxis(img, -1 * tOffset + fOffset, 0) />
		<cfset my.falseList.draw(img) />
		<cfset imageTranslateDrawingAxis(img, -1 * fOffset, -1 * (BOX_HEIGHT + TOP_MARGIN)) />

		<!--- bottom arrow --->
		<cfif NOT wasTrueTerminated>
			<cfset my.trueList.resetColors(img) />
			<cfset imageDrawLine(img, tOffset, min(BOX_HEIGHT + TOP_MARGIN + t.height, m.height - BOTTOM_MARGIN), tOffset, m.height) />
			<cfset imageDrawLine(img, tOffset, m.height, -1 * shift, m.height) />
		</cfif>
		<cfif NOT wasFalseTerminated>
			<cfset my.falseList.resetColors(img) />
			<cfset imageDrawLine(img, fOffset, min(BOX_HEIGHT + TOP_MARGIN + f.height, m.height - BOTTOM_MARGIN), fOffset, m.height) />
			<cfset imageDrawLine(img, -1 * shift, m.height, fOffset, m.height) />
		</cfif>
		
		<cfset setFlowTerminated(my.trueList.isFlowTerminated() AND my.falseList.isFlowTerminated()) />
		<cfset resetColors(img) />
		<!--- this deliberately overhangs the bottom edge by 10 points --->
		<cfset imageDrawLine(img, -1 * shift, m.height, -1 * shift, m.height + 10) />
		<cfset imageTranslateDrawingAxis(img, -1 * shift, 0) />
	</cffunction>



	<cffunction name="measureInternal" access="private" output="false" returntype="struct">
		<cfset var t = my.trueList.measure() />
		<cfset var f = my.falseList.measure() />
		<cfset var s = {
			width = LEFT_MARGIN + max(BOX_WIDTH, t.width) + CENTER_SPACE + max(BOX_WIDTH, f.width) + RIGHT_MARGIN,
			height = BOX_HEIGHT + TOP_MARGIN + max(t.height, f.height) + BOTTOM_MARGIN
		} />
		<cfreturn s />
	</cffunction>

</cfcomponent>