<cfcomponent output="false">

	<cfset BOX_WIDTH = 100 />
	<cfset BOX_HEIGHT = 70 />
	
	<cfset NORMAL_COLOR = "000000" />
	<cfset DEAD_COLOR = "cccccc" />
	
	<cfset variables.my.uuid = createUuid() />
	<cfset variables.my.graphics = imageGetBufferedImage(imageNew("", 10, 10, "rgb")).getGraphics() />

	<cffunction name="init" access="public" output="false" returntype="shape">
		<cfargument name="verb" type="struct" required="false" />
		<cfif structKeyExists(arguments, "verb")>
			<cfset variables.my.verb = verb />
		<cfelse>
			<cfset variables.my.verb = {} />
		</cfif>
		<cfset my.flowTerminated = false />
		<cfreturn this />
	</cffunction>
	


	<cffunction name="getLabel" access="public" output="false" returntype="string">
		<cfscript>
			try {
				switch (my.verb.type) {
					case "do":
						if (my.verb.action CONTAINS ".") {
							return "do '" & my.verb.action & "'";
						} else {
							return "do '" & my.verb.circuit & "." & my.verb.action & "'";
						}
					case "prefuseaction":
						return "preFA '" & my.verb.circuit & "'";
					case "postfuseaction":
						return "postFA '" & my.verb.circuit & "'";
					case "preprocess":
						return "global preprocess";
					case "postprocess":
						return "global postprocess";
					case "set":
					case "xfa":
						return my.verb.type & " '" & my.verb.attributes.name & "'";
					case "include":
					case "util.include":
						return my.verb.type & " '" & my.verb.attributes.template & "'";
					case "invoke":
					case "util.invoke":
						return my.verb.type & " '" & my.verb.attributes.object & "." & my.verb.attributes.methodCall & "'";
				}
			} catch (any e) {
				return e.message;
				// whatever
			}
			return my.verb.type;
		</cfscript>
	</cffunction>



	<cffunction name="getLabelWidth" access="public" output="false" returntype="numeric">
		<cfreturn getTextWidth(getLabel()) + 20 />
	</cffunction>



	<cffunction name="isFlowTerminated" access="public" output="false" returntype="boolean">
		<cfreturn my.flowTerminated />
	</cffunction>



	<cffunction name="setFlowTerminated" access="public" output="false" returntype="void">
		<cfargument name="flowTerminated" type="boolean" required="true" />
		<cfset my.flowTerminated = flowTerminated />
	</cffunction>



	<cffunction name="terminatesFlow" access="public" output="false" returntype="boolean">
		<cfreturn false /><!--- subclasses can override this --->
	</cffunction>



	<cffunction name="getTextHeight" access="public" output="false" returntype="numeric">
		<cfargument name="txt" type="string" default="my string" />
		<!--- we can't ask for the Graphics of the active image, because it resets the axis translation --->
		<cfreturn my.graphics.getFontMetrics().getStringBounds(txt, my.graphics).getHeight() />
	</cffunction>



	<cffunction name="getTextWidth" access="public" output="false" returntype="numeric">
		<cfargument name="txt" type="string" required="true" />
		<!--- we can't ask for the Graphics of the active image, because it resets the axis translation --->
		<cfreturn my.graphics.getFontMetrics().getStringBounds(txt, my.graphics).getWidth() />
	</cffunction>



	<cffunction name="invalidate" access="public" output="false" returntype="void">
		<cflock name="#my.uuid#.measured" type="exclusive" timeout="2" throwontimeout="true">
			<cfset structDelete(my, "measured") />
		</cflock>
	</cffunction>



	<cffunction name="measure" access="public" output="false" returntype="struct">
		<cflock name="#my.uuid#.measured" type="exclusive" timeout="3" throwontimeout="true">
			<cfif NOT structKeyExists(my, "measured")>
				<cfset my.measured = measureInternal() />
			</cfif>
			<cfreturn my.measured />
		</cflock>
	</cffunction>
	


	<cffunction name="measureInternal" access="private" output="false" returntype="struct">
		<cfthrow type="UnsupportedOperationException"
			message="Subclasses of 'shape' MUST implement measureInternal" />
	</cffunction>



	<cffunction name="resetColors" access="package" output="false" returntype="void">
		<cfargument name="img" type="any" required="true" />
		<cfset imageSetDrawingColor(img, iif(isFlowTerminated(), de(DEAD_COLOR), de(NORMAL_COLOR))) />
	</cffunction>



	<cfset System = createObject("java", "java.lang.System") />
	<cffunction name="msg" access="public" output="false" returntype="void">
		<cfargument name="msg" type="string" default="" />
		<cfset System.out.println(msg) />
	</cffunction>

</cfcomponent>