<cfcomponent extends="shape" output="false">
	
	<cfset ARROW_LENGTH = 30 />
	
	<cffunction name="init" access="public" output="false" returntype="shapelist">
		<cfargument name="verbs" type="array" required="false" />
		<cfset variables.my.shapes = [] />
		<cfif structKeyExists(arguments, "verbs")>
			<cfset addVerbs(verbs) />
		</cfif>
		<cfset my.flowTerminated = false />
		<cfreturn this />
	</cffunction>
	


	<cffunction name="addVerb" access="public" output="false" returntype="void">
		<cfargument name="v" type="struct" required="true" />
		<cfset var s = "" />
		<cfset s = "box" />
		<cfif v.type EQ "if">
			<cfset s = "fork" />
		<cfelseif v.type EQ "loop">
			<cfset s = "loop" />
		<cfelseif listFindNoCase("relocate,cf:abort", v.type) GT 0>
			<cfset s = "termination" />
		<cfelseif listFindNoCase("unknowncircuit,unknownfuseaction", v.type) GT 0>
			<cfset s = "error" />
		<cfelseif listFindNoCase("do,preprocess,postprocess,prefuseaction,postfuseaction,view.savecontent,view.div", v.type) GT 0>
			<cfset s = "inline" />
		</cfif>
		<cfset addShape(createObject("component", s).init(v)) />
	</cffunction>



	<cffunction name="addVerbs" access="public" output="false" returntype="void">
		<cfargument name="verbs" type="array" required="false" />
		<cfset var v = "" />
		<cfloop array="#verbs#" index="v">
			<cfset addVerb(v) />
		</cfloop>
	</cffunction>



	<cffunction name="addShape" access="public" output="false" returntype="void">
		<cfargument name="shape" type="shape" required="true" />
		<cfset arrayAppend(my.shapes, shape) />
		<cfset invalidate() />
	</cffunction>



	<cffunction name="addShapes" access="public" output="false" returntype="void">
		<cfargument name="shapes" type="array" required="false" />
		<cfset var s = "" />
		<cfloop array="#shapes#" index="s">
			<cfset addShape(s) />
		</cfloop>
	</cffunction>



	<cffunction name="draw" access="public" output="false" returntype="void">
		<cfargument name="img" type="any" required="true" />
		<cfset var m = measure() />
		<cfset var shape = "" />
		<cfset var shift = 0 />
		<cfset var i = 0 />
<!---
<cfset msg("list (#m.width# x #m.height#)") />
<cfset imageSetDrawingColor(img, "99ff99") />
<cfset imageDrawRect(img, m.width / -2, 0, m.width, m.height) />
--->
		<cfloop array="#my.shapes#" index="shape">
			<cfset i += 1 />
			<cfif isFlowTerminated()>
				<cfset shape.setFlowTerminated(true) />
			</cfif>
			<cfif i GT 1>
				<!--- draw the arrow from the previous down to this one --->
				<cfset resetColors(img) />
				<cfset imageDrawLine(img, 0, -1 * ARROW_LENGTH, 0, 0) />
				<cfset imageDrawLine(img, 0, 0, -10, -10) />
				<cfset imageDrawLine(img, 0, 0, 10, -10) />
			</cfif>
			<cfset m = shape.measure() />
			<cfset shape.draw(img) />
			<cfset imageTranslateDrawingAxis(img, 0, m.height + ARROW_LENGTH) />
			<cfset shift += m.height + ARROW_LENGTH />
			<cfif shape.isFlowTerminated() OR shape.terminatesFlow()>
				<cfset setFlowTerminated(true) />
			</cfif>
		</cfloop>
		<cfset imageTranslateDrawingAxis(img, 0, -1 * shift) />
	</cffunction>



	<cffunction name="getSize" access="public" output="false" returntype="numeric">
		<cfreturn arrayLen(my.shapes) />
	</cffunction>



	<cffunction name="isEmpty" access="public" output="false" returntype="boolean">
		<cfreturn arrayLen(my.shapes) EQ 0 />
	</cffunction>



	<cffunction name="measureInternal" access="private" output="false" returntype="struct">
		<cfset var s = {
			width = 0,
			height = 0
		} />
		<cfset var shape = "" />
		<cfset var m = "" />
		<cfset var i = 0 />
		<cfloop array="#my.shapes#" index="shape">
			<cfset i += 1 />
			<cfset m = shape.measure() />
			<cfset s.height += m.height />
			<cfset s.width = max(s.width, m.width) />
			<cfif i GT 1>
				<cfset s.height += ARROW_LENGTH />
			</cfif>
		</cfloop>
		<cfreturn s />
	</cffunction>

</cfcomponent>